@@ -28,18 +28,17 @@
 #include <gtk/gtk.h>
 #include <hildon/hildon-window.h>
 #include <hildon/hildon-find-toolbar.h>
+#include <hildon/hildon-helper.h>
 #ifdef HILDON_HAS_APP_MENU
 #include <hildon/hildon-text-view.h>
 #include <hildon/hildon-pannable-area.h>
 #endif
 
 #include "callbacks.h"
-
-#include "serializer.h"
-#include "deserializer.h"
 #include "metadata.h"
 #include "interface.h"
 #include "app_data.h"
+#include "settings.h"
 
 
 static void initialize_tags(GtkTextBuffer *buffer) {
@@ -96,6 +95,119 @@ set_item_label(GtkContainer *item, const gchar *open_tag, const gchar *text, con
 	g_free(string);
 }
 
+static void
+on_orientation_changed(GdkScreen *screen, GHashTable *hash)
+{
+	AppData *app_data = app_data_get();
+
+	GtkWidget *toolbar = g_hash_table_lookup(hash, "toolbar");
+	GtkWidget *menu_open = g_hash_table_lookup(hash, "menu_open");
+
+	app_data->portrait = is_portrait_mode();
+
+	if (app_data->portrait) {
+		gtk_widget_hide(toolbar);
+		gtk_widget_show(menu_open);
+	} else {
+		gtk_widget_show(toolbar);
+		gtk_widget_hide(menu_open);
+	}
+}
+
+static void
+on_scrollbar_settings_changed(GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
+{
+	GtkScrolledWindow *win = GTK_SCROLLED_WINDOW(user_data);
+
+	SettingsScrollbarSize size = gconf_value_get_int(entry->value);
+	if (size == SETTINGS_SCROLLBAR_SIZE_BIG) {
+		hildon_helper_set_thumb_scrollbar(win, TRUE);
+	} else {
+		hildon_helper_set_thumb_scrollbar(win, FALSE);
+	}
+}
+
+static void
+on_background_color_changed(GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
+{
+	GtkWidget *view = GTK_WIDGET(user_data);
+	GdkColor color;
+
+	const gchar *hex_color = gconf_value_get_string(entry->value);
+	gdk_color_parse(hex_color, &color);
+
+	gtk_widget_modify_base(GTK_WIDGET(view), GTK_STATE_NORMAL, &color);
+
+	/* Small stripe on the right side only */
+	/*gtk_widget_modify_bg(GTK_WIDGET(window), GTK_STATE_NORMAL, &color);*/
+}
+
+static void
+on_text_color_changed(GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
+{
+	GtkWidget *view = GTK_WIDGET(user_data);
+	GdkColor color;
+
+	const gchar *hex_color = gconf_value_get_string(entry->value);
+	gdk_color_parse(hex_color, &color);
+
+	g_printerr("Text color changed \n");
+
+	gtk_widget_modify_text(GTK_WIDGET(view), GTK_STATE_NORMAL, &color);
+}
+
+static void
+on_link_color_changed(GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
+{
+	if (user_data == NULL || !GTK_IS_TEXT_VIEW(user_data)) {
+		g_printerr("ERROR: on_link_color_changed(): user_data is not a GtkTextView.");
+		return;
+	}
+	GtkTextView *view = GTK_TEXT_VIEW(user_data);
+	GtkTextBuffer *buffer = gtk_text_view_get_buffer(view);
+	GtkTextTag *title = gtk_text_tag_table_lookup(buffer->tag_table, "_title");
+	GtkTextTag *link = gtk_text_tag_table_lookup(buffer->tag_table, "link:internal");
+	const gchar *hex_color = gconf_value_get_string(entry->value);
+
+	g_object_set(title, "foreground", hex_color, NULL);
+	g_object_set(link, "foreground", hex_color, NULL);
+}
+
+static void
+on_use_custom_colors_changed(GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
+{
+	GtkTextView *view = GTK_TEXT_VIEW(user_data);
+
+	if (gconf_value_get_bool(entry->value)) {
+		/* We just emmit the changed signal on gconf, so the UI will update itself */
+		AppData *app_data = app_data_get();
+		gconf_client_notify(app_data->client, SETTINGS_BACKGROUND_COLOR);
+		gconf_client_notify(app_data->client, SETTINGS_TEXT_COLOR);
+		gconf_client_notify(app_data->client, SETTINGS_LINK_COLOR);
+
+	} else {
+		/* Reset to default colors */
+		GtkTextBuffer *buffer = gtk_text_view_get_buffer(view);
+		GtkTextTag *title = gtk_text_tag_table_lookup(buffer->tag_table, "_title");
+		GtkTextTag *link = gtk_text_tag_table_lookup(buffer->tag_table, "link:internal");
+		g_object_set(title, "foreground", "blue", NULL);
+		g_object_set(link, "foreground", "blue", NULL);
+
+		gtk_widget_modify_base(GTK_WIDGET(view), GTK_STATE_NORMAL, NULL);
+		gtk_widget_modify_text(GTK_WIDGET(view), GTK_STATE_NORMAL, NULL);
+	}
+}
+
+static void
+on_font_size_changed(GConfClient *client, guint cnxn_id, GConfEntry *entry, gpointer user_data)
+{
+	GtkWidget *textview = GTK_WIDGET(user_data);
+	PangoFontDescription *font = pango_font_description_new();
+	pango_font_description_set_size(font, gconf_value_get_int(entry->value));
+	gtk_widget_modify_font(GTK_WIDGET(textview), font);
+	pango_font_description_free(font);
+}
+
 /*
  * I'm not sure what is better:
  * 1) Declaring all as GtkWidget, so that Fremantle and Diablo code can share these.
@@ -109,6 +221,7 @@ GtkWidget* create_mainwin(Note *note) {
 	GtkWidget *vbox1;
 	GtkWidget *main_menu;
 	GtkWidget *text_style_menu;
+	GdkScreen *screen;
 
 	GtkWidget *menu_new;
 	GtkWidget *menu_bold;
@@ -117,6 +230,7 @@ GtkWidget* create_mainwin(Note *note) {
 	GtkWidget *menu_highlight;
 	GtkWidget *menu_fixed;
 	GtkWidget *menu_bullets;
+	GtkWidget *menu_settings;
 	GtkWidget *menu_quit;
 	GtkWidget *menu_text_style;
 	GtkWidget *menu_inc_indent;
@@ -125,7 +239,7 @@ GtkWidget* create_mainwin(Note *note) {
 	GtkWidget *menu_font_normal;
 	GtkWidget *menu_font_large;
 	GtkWidget *menu_font_huge;
-	GtkWidget *menu_find;
+	GtkWidget *menu_open;
 
 	GtkWidget *toolbar;
 	GtkWidget *find_bar;
@@ -143,6 +257,7 @@ GtkWidget* create_mainwin(Note *note) {
 
 	GtkAction *action_new;
 	GtkAction *action_delete;
+	GtkAction *action_settings;
 	GtkAction *action_quit;
 	GtkAction *action_notes;
 	GtkAction *action_bold;
@@ -177,6 +292,8 @@ GtkWidget* create_mainwin(Note *note) {
 	mainwin = hildon_window_new();
 	gtk_window_set_title(GTK_WINDOW(mainwin), "Conboy");
 
+	screen = gdk_screen_get_default();
+
 	accel_group = gtk_accel_group_new();
 	gtk_window_add_accel_group(GTK_WINDOW(mainwin), accel_group);
 	g_object_unref(accel_group);
@@ -195,7 +312,8 @@ GtkWidget* create_mainwin(Note *note) {
 	action_inc_indent = GTK_ACTION(gtk_action_new("inc_indent", _("Increase Indent"), NULL, GTK_STOCK_INDENT));
 	action_link = GTK_ACTION(gtk_action_new("link", _("Link"), NULL, GTK_STOCK_REDO));
 	action_new = GTK_ACTION(gtk_action_new("new", _("New Note"), NULL, NULL));
-	action_notes = GTK_ACTION(gtk_action_new("open", _("Open"), NULL, GTK_STOCK_OPEN));
+	action_notes = GTK_ACTION(gtk_action_new("open", _("Open Note"), NULL, GTK_STOCK_OPEN));
+	action_settings = GTK_ACTION(gtk_action_new("settings", _("Settings"), NULL, NULL));
 	action_quit = GTK_ACTION(gtk_action_new("quit", _("Close All Notes"), NULL, NULL));
 	action_italic = GTK_ACTION(gtk_toggle_action_new("italic", _("Italic"), NULL, GTK_STOCK_ITALIC));
 	action_strike = GTK_ACTION(gtk_toggle_action_new("strikethrough", _("Strikeout"), NULL, NULL));
@@ -353,15 +471,27 @@ GtkWidget* create_mainwin(Note *note) {
 #ifdef HILDON_HAS_APP_MENU
 	main_menu = hildon_app_menu_new();
 
-	menu_quit = gtk_button_new();
 	menu_new = gtk_button_new();
+	menu_open = gtk_button_new();
+	menu_settings = gtk_button_new();
+	menu_quit = gtk_button_new();
 
 	gtk_action_connect_proxy(action_new, menu_new);
+	gtk_action_connect_proxy(action_notes, menu_open);
+	gtk_action_connect_proxy(action_settings, menu_settings);
 	gtk_action_connect_proxy(action_quit, menu_quit);
 
 	hildon_app_menu_append(HILDON_APP_MENU(main_menu), GTK_BUTTON(menu_new));
+	hildon_app_menu_append(HILDON_APP_MENU(main_menu), GTK_BUTTON(menu_open));
+	hildon_app_menu_append(HILDON_APP_MENU(main_menu), GTK_BUTTON(menu_settings));
 	hildon_app_menu_append(HILDON_APP_MENU(main_menu), GTK_BUTTON(menu_quit));
 
+	if (app_data->portrait) {
+		gtk_widget_hide(menu_settings);
+	} else {
+		gtk_widget_hide(menu_open);
+	}
+
 	hildon_window_set_app_menu(HILDON_WINDOW(mainwin), HILDON_APP_MENU(main_menu));
 
 #else
@@ -370,16 +500,14 @@ GtkWidget* create_mainwin(Note *note) {
 	menu_new = gtk_action_create_menu_item(action_new);
 	menu_text_style = gtk_menu_item_new_with_label(_("Text Style"));
 	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_text_style), text_style_menu);
-	menu_find = gtk_action_create_menu_item(action_find);
+	menu_settings = gtk_action_create_menu_item(action_settings);
 	menu_quit = gtk_action_create_menu_item(action_quit);
 
 	gtk_menu_shell_append(GTK_MENU_SHELL(main_menu), menu_new);
 	gtk_menu_shell_append(GTK_MENU_SHELL(main_menu), gtk_separator_menu_item_new());
 	gtk_menu_shell_append(GTK_MENU_SHELL(main_menu), menu_text_style);
-	/*
 	gtk_menu_shell_append(GTK_MENU_SHELL(main_menu), gtk_separator_menu_item_new());
-	gtk_menu_shell_append(GTK_MENU_SHELL(main_menu), menu_find);
-	*/
+	gtk_menu_shell_append(GTK_MENU_SHELL(main_menu), menu_settings);
 	gtk_menu_shell_append(GTK_MENU_SHELL(main_menu), gtk_separator_menu_item_new());
 	gtk_menu_shell_append(GTK_MENU_SHELL(main_menu), menu_quit);
 
@@ -394,7 +522,11 @@ GtkWidget* create_mainwin(Note *note) {
 	/****/
 	button_dec_indent = gtk_action_create_tool_item(action_dec_indent);
 	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), GTK_TOOL_ITEM(button_dec_indent), -1);
+#ifdef HILDON_HAS_APP_MENU
+	gtk_widget_set_size_request(GTK_WIDGET(button_dec_indent), 105, -1);
+#else
 	gtk_widget_set_size_request(GTK_WIDGET(button_dec_indent), 85, -1);
+#endif
 
 	button_inc_indent = gtk_action_create_tool_item(action_inc_indent);
 	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), GTK_TOOL_ITEM(button_inc_indent), -1);
@@ -422,9 +554,12 @@ GtkWidget* create_mainwin(Note *note) {
 	button_notes = gtk_action_create_tool_item(action_notes);
 	gtk_toolbar_insert(GTK_TOOLBAR(toolbar), GTK_TOOL_ITEM(button_notes), -1);
 
-
 	gtk_widget_show_all(toolbar);
 
+	if (app_data->portrait) {
+		gtk_widget_hide(toolbar);
+	}
+
 	hildon_window_add_toolbar(HILDON_WINDOW(mainwin), GTK_TOOLBAR(toolbar));
 	/* TODO: Maybe we can use one intance of the toolbar for all windows. */
 	/*
@@ -462,18 +597,47 @@ GtkWidget* create_mainwin(Note *note) {
 	gtk_widget_show(textview);
 	gtk_container_add(GTK_CONTAINER (scrolledwindow1), textview);
 
-	/* Font */
-	font = pango_font_description_new();
-	pango_font_description_set_size(font, app_data->font_size);
-	gtk_widget_modify_font(GTK_WIDGET(textview), font);
-
-
-	/* Enable support for tap and hold on the textview */
-	gtk_widget_tap_and_hold_setup(textview, NULL, NULL, 0);
 
 	/* Create basic set of tags */
 	initialize_tags(buffer);
 
+	/* Set initial font size */
+	font = pango_font_description_new();
+	pango_font_description_set_size(font, settings_load_font_size());
+	gtk_widget_modify_font(GTK_WIDGET(textview), font);
+	pango_font_description_free(font);
+
+	/* Set initial scrollbar size */
+	if (settings_load_scrollbar_size() == SETTINGS_SCROLLBAR_SIZE_BIG) {
+		hildon_helper_set_thumb_scrollbar(GTK_SCROLLED_WINDOW(scrolledwindow1), TRUE);
+	}
+
+	/* Set initial colors */
+	if (settings_load_use_costum_colors()) {
+		GdkColor color;
+		GtkTextTag *title = gtk_text_tag_table_lookup(buffer->tag_table, "_title");
+		GtkTextTag *link = gtk_text_tag_table_lookup(buffer->tag_table, "link:internal");
+
+		/* Background color */
+		settings_load_color(&color, SETTINGS_COLOR_TYPE_BACKGROUND);
+		gtk_widget_modify_base(textview, GTK_STATE_NORMAL, &color);
+
+		/* Text color */
+		settings_load_color(&color, SETTINGS_COLOR_TYPE_TEXT);
+		gtk_widget_modify_text(textview, GTK_STATE_NORMAL, &color);
+
+		/* Link color */
+		settings_load_color(&color, SETTINGS_COLOR_TYPE_LINKS);
+		g_object_set(title, "foreground-gdk", &color, NULL);
+		g_object_set(link, "foreground-gdk", &color, NULL);
+	}
+
+
+	/* Enable support for tap and hold on the textview */
+	/* TODO: This is deactivated for now, because it's not included in upstream Gtk
+	 * and therefore does not work on exist in ubuntu */
+	/*gtk_widget_tap_and_hold_setup(textview, NULL, NULL, 0);*/
+
 	/* Save for later usage */
 	ui->window = HILDON_WINDOW(mainwin);
 	ui->view = GTK_TEXT_VIEW(textview);
@@ -536,6 +700,10 @@ GtkWidget* create_mainwin(Note *note) {
 			G_CALLBACK(on_quit_button_clicked),
 			NULL);
 
+	g_signal_connect(action_settings, "activate",
+			G_CALLBACK(on_settings_button_clicked),
+			mainwin);
+
 	g_signal_connect(action_inc_indent, "activate",
 			G_CALLBACK(on_inc_indent_button_clicked),
 			buffer);
@@ -614,6 +782,25 @@ GtkWidget* create_mainwin(Note *note) {
 			G_CALLBACK(on_find_bar_close),
 			ui);
 
+	/* Listen to changes in the settings */
+	/* TODO: Use an array instead of the list */
+	ui->listeners = g_list_prepend(ui->listeners, GINT_TO_POINTER(gconf_client_notify_add(app_data->client, SETTINGS_SCROLLBAR_SIZE, on_scrollbar_settings_changed, scrolledwindow1, NULL, NULL)));
+	ui->listeners = g_list_prepend(ui->listeners, GINT_TO_POINTER(gconf_client_notify_add(app_data->client, SETTINGS_BACKGROUND_COLOR, on_background_color_changed, textview, NULL, NULL)));
+	ui->listeners = g_list_prepend(ui->listeners, GINT_TO_POINTER(gconf_client_notify_add(app_data->client, SETTINGS_TEXT_COLOR, on_text_color_changed, textview, NULL, NULL)));
+	ui->listeners = g_list_prepend(ui->listeners, GINT_TO_POINTER(gconf_client_notify_add(app_data->client, SETTINGS_LINK_COLOR, on_link_color_changed, textview, NULL, NULL)));
+	ui->listeners = g_list_prepend(ui->listeners, GINT_TO_POINTER(gconf_client_notify_add(app_data->client, SETTINGS_USE_CUSTOM_COLORS, on_use_custom_colors_changed, textview, NULL, NULL)));
+	ui->listeners = g_list_prepend(ui->listeners, GINT_TO_POINTER(gconf_client_notify_add(app_data->client, SETTINGS_FONT_SIZE, on_font_size_changed, textview, NULL, NULL)));
+
+
+	/* TODO: When restructuring the UI, don't use the hash map anymore */
+	GHashTable *hash = g_hash_table_new(g_str_hash, g_str_equal);
+	g_hash_table_insert(hash, "toolbar", toolbar);
+	g_hash_table_insert(hash, "menu_new", menu_new);
+	g_hash_table_insert(hash, "menu_open", menu_open);
+	g_signal_connect((gpointer)screen, "size-changed",
+			G_CALLBACK(on_orientation_changed),
+			hash);
+
 	link_internal_tag = gtk_text_tag_table_lookup(buffer->tag_table, "link:internal");
 	g_signal_connect ((gpointer) link_internal_tag, "event",
 			G_CALLBACK (on_link_internal_tag_event),
