@@ -1,5 +1,5 @@
 /* This file is part of Conboy.
- * 
+ *
  * Copyright (C) 2009 Cornelius Hald
  *
  * Conboy is free software: you can redistribute it and/or modify
@@ -59,11 +59,14 @@ const gchar* get_bullet_by_depth_tag(GtkTextTag *tag) {
 
 
 gchar* get_uuid()
-{	
+{
 	gchar *content;
+	gchar *result;
 	g_file_get_contents("/proc/sys/kernel/random/uuid", &content, NULL, NULL);
 	g_strchomp(content);
-	return g_strdup(content);
+	result = g_strconcat(content, "\0", NULL);
+	g_free(content);
+	return result;
 }
 
 const gchar* note_get_new_filename(const gchar *uuid)
@@ -76,44 +79,44 @@ const gchar* note_get_new_filename(const gchar *uuid)
 /**
  * Returns the given time as iso8601 formatted string.
  * Example: 2009-03-24T13:16:42.0000000+01:00
- * 
+ *
  * This method is needed, because the glib function
  * g_time_val_to_iso8601() does only produce the short format without
  * milliseconds. E.g. 2009-04-17T13:14:52Z
  */
 const gchar* get_time_in_seconds_as_iso8601(time_t time_in_seconds) {
-	
+
 	gchar      time_string[40];
 	gchar     *minutes;
 	gchar     *text_pointer;
 	struct tm *local_time;
 	gchar     *first_part;
 	gchar     *result;
-	
+
 	local_time = localtime(&time_in_seconds);
-	
+
 	/* Milliseconds are always 0 and timezone is +0100 -> should be +01:00 */
 	strftime(time_string, 40, "%Y-%m-%dT%H:%M:%S.0000000%z", local_time);
-	
+
 	/* Save minutes, add colon, add minutes */
 	text_pointer = (gchar*) &time_string;
 	text_pointer = text_pointer + 30;
-	
+
 	minutes = g_strndup(text_pointer, 2);
 	first_part = g_strndup(time_string, 30);
 	result = g_strconcat(first_part, ":", minutes, NULL);
-	
+
 	g_free(minutes);
 	g_free(first_part);
-	
-	return result;	
+
+	return result;
 }
 
 
 /**
  * Returns the current time as string formattet as in iso8601.
  * Example: 2009-03-24T13:16:42.0000000+01:00
- * 
+ *
  * This method is needed, because the glib function
  * g_time_val_to_iso8601() does only produce the short format without
  * milliseconds. E.g. 2009-04-17T13:14:52Z
@@ -123,17 +126,32 @@ const gchar* get_current_time_in_iso8601() {
 }
 
 time_t get_iso8601_time_in_seconds(const gchar *time_string) {
-	
+
 	time_t result;
 	struct tm local_time;
-	
+
 	/* Looks like it's ok that the colon is used with the timezone,
 	 * if there will be trouble, remove the colon first. See
 	 * get_current_time_in_iso8601() */
-	
+
 	strptime(time_string, "%Y-%m-%dT%H:%M:%S.0000000%z", &local_time);
 	result = mktime(&local_time);
 	return result;
 }
 
+gboolean is_portrait_mode()
+{
+	GdkScreen *screen = gdk_screen_get_default();
+	int width = gdk_screen_get_width(screen);
+	int height = gdk_screen_get_height(screen);
+
+	g_printerr("Screen size: %i x %i \n", width, height);
+
+	/* Hopefully we don't have a square screen ;) */
+	if (width > height) {
+		return FALSE;
+	} else {
+		return TRUE;
+	}
+}
 
